/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.control;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;

import de.tum.in.wwwbruegge.asteroids.solidbodies.SpaceShuttle;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SpaceShuttleSubscriber;

/**
 * @author Timo Wolf
 */
public class SpaceShuttleController implements SpaceShuttleSubscriber {

	private Action speedUpAction;

	private Action speedDownAction;

	private Action turnRightAction;

	private Action turnLeftAction;

	private Action fireRocketAction;

	private ArrayList<Action> actionList;

	private SpaceShuttle spaceShuttle;

	public SpaceShuttleController(SpaceShuttle spaceShuttle) {
		this.spaceShuttle = spaceShuttle;
		this.spaceShuttle.subscribe(this);

		actionList = new ArrayList<Action>();

		speedUpAction = new AbstractAction("speed up") {
			public void actionPerformed(ActionEvent e) {
				SpaceShuttleController.this.spaceShuttle.incrementSpeed();
			}
		};
		speedUpAction.putValue(Action.ACCELERATOR_KEY,
				getMenuShortcut(KeyEvent.VK_UP));

		speedDownAction = new AbstractAction("speed down") {

			public void actionPerformed(ActionEvent e) {
				SpaceShuttleController.this.spaceShuttle.decrementSpeed();
			}
		};
		speedDownAction.putValue(Action.ACCELERATOR_KEY,
				getMenuShortcut(KeyEvent.VK_DOWN));

		turnLeftAction = new AbstractAction("left") {
			public void actionPerformed(ActionEvent e) {
				SpaceShuttleController.this.spaceShuttle.turnLeft();
			}
		};
		turnLeftAction.putValue(Action.ACCELERATOR_KEY,
				getMenuShortcut(KeyEvent.VK_LEFT));

		turnRightAction = new AbstractAction("right") {
			public void actionPerformed(ActionEvent e) {
				SpaceShuttleController.this.spaceShuttle.turnRight();
			}
		};
		turnRightAction.putValue(Action.ACCELERATOR_KEY,
				getMenuShortcut(KeyEvent.VK_RIGHT));

		fireRocketAction = new AbstractAction("fire rocket") {
			public void actionPerformed(ActionEvent e) {
				SpaceShuttleController.this.spaceShuttle.fireRocket();
			}
		};
		fireRocketAction.putValue(Action.ACCELERATOR_KEY,
				getMenuShortcut(KeyEvent.VK_SPACE));

		this.actionList.add(speedUpAction);
		this.actionList.add(speedDownAction);
		this.actionList.add(turnLeftAction);
		this.actionList.add(turnRightAction);
		this.actionList.add(fireRocketAction);
	}

	public Action getSpeedUpAction() {
		return speedUpAction;
	}

	public Action getSpeedDownAction() {
		return speedDownAction;
	}

	public Action getTurnRightAction() {
		return turnRightAction;
	}

	public Action getTurnLeftAction() {
		return turnLeftAction;
	}

	public Action getFireRocketAction() {
		return fireRocketAction;
	}

	public ArrayList getActions() {
		return actionList;
	}

	public void update() {
		if (spaceShuttle.getSpeed() + SpaceShuttle.ACCELARATION > spaceShuttle
				.getMaximumSpeed()) {
			speedUpAction.setEnabled(false);
		} else {
			speedUpAction.setEnabled(true);
		}

		if (spaceShuttle.getSpeed() - SpaceShuttle.ACCELARATION < spaceShuttle
				.getMinimumSpeed()) {
			speedDownAction.setEnabled(false);
		} else {
			speedDownAction.setEnabled(true);
		}
	}

	private static KeyStroke getMenuShortcut(int keyCode) {
		return KeyStroke.getKeyStroke(keyCode, 0, false);
	}
}
