/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.solidbodies;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;

import de.tum.in.wwwbruegge.asteroids.ui.GameBoard;

/**
 * @author Timo Wolf
 */
public class SpaceShuttle extends SolidBody {

	public static Image DEFAULT_BODY;

	private static int ROTATION_ANGLE = 10;

	public final static int ACCELARATION = 40;

	private ArrayList<SpaceShuttleSubscriber> subscribers;

	public SpaceShuttle(Point position, int direction) {
		super(position, direction);
		subscribers = new ArrayList<SpaceShuttleSubscriber>();
		setSpeed(3);
		notifySpaceShuttleSubscribers();
	}

	public void initiate(Point position, int direction) {
		setBody(DEFAULT_BODY);
		this.position = position;
		setDirection(direction);
		setSpeed(3);
		notifySpaceShuttleSubscribers();
	}

	public void incrementSpeed() {
		if (getSpeed() + ACCELARATION <= getMaximumSpeed()) {
			setSpeed(getSpeed() + ACCELARATION);
		}
	}

	public void decrementSpeed() {
		if (getSpeed() - ACCELARATION >= getMinimumSpeed()) {
			setSpeed(getSpeed() - ACCELARATION);
		}
	}

	public void turnRight() {
		int tmp = getDirection() + ROTATION_ANGLE;
		if (tmp > 360) {
			setDirection(ROTATION_ANGLE);
		} else {
			setDirection(tmp);
		}
	}

	public void turnLeft() {
		int tmp = getDirection() - ROTATION_ANGLE;
		if (tmp < 0) {
			setDirection(360 - ROTATION_ANGLE);
		} else {
			setDirection(tmp);
		}
	}

	public void fireRocket() {
		int width = getSize().width;
		int height = getSize().height;

		int delta_x = (int) (((float) width) * (float) Math.sin(Math
				.toRadians(getDirection())));
		int delta_y = (int) (((float) height) * (float) Math.cos(Math
				.toRadians(getDirection())));
		Point p = new Point(position.x + width / 2 + delta_x, position.y
				- height / 2 + delta_y);
		Rocket rocket = new Rocket(p, getDirection());
		GameBoard.getInstance().addSolidBody(rocket);
	}

	public synchronized void updatePosition(int max_x, int max_y) {
		super.updatePosition(max_x, max_y);
		notifySpaceShuttleSubscribers();
	}

	protected void setSpeed(int speed) {
		super.setSpeed(speed);
		notifySpaceShuttleSubscribers();
	}

	protected void setDirection(int direction) {
		super.setDirection(direction);
		notifySpaceShuttleSubscribers();
	}

	public void subscribe(SpaceShuttleSubscriber subscriber) {
		if (subscriber != null && !this.subscribers.contains(subscriber)) {
			this.subscribers.add(subscriber);
		}
	}

	public void unsubscribe(SpaceShuttleSubscriber subscriber) {
		if (subscriber != null && this.subscribers.contains(subscriber)) {
			this.subscribers.remove(subscriber);
		}
	}

	public synchronized void notifySpaceShuttleSubscribers() {
		if (subscribers != null) {
			Iterator iter = subscribers.iterator();
			while (iter.hasNext()) {
				SpaceShuttleSubscriber subscriber = (SpaceShuttleSubscriber) iter
						.next();
				subscriber.update();
			}
		}
	}
}
